package internalgengo

import (
	"google.golang.org/protobuf/compiler/protogen"
	"google.golang.org/protobuf/reflect/protoreflect"
)

type structField string

func (f structField) String() string {
	return string(f)
}

func (f structField) PointerTo() structField {
	return "*" + f
}

func (f structField) SliceOf() structField {
	return "[]" + f
}

func fieldType(g *protogen.GeneratedFile, field *protogen.Field) structField {
	var typ structField
	switch field.Desc.Kind() {
	case protoreflect.BoolKind:
		typ = "bool"
	case protoreflect.EnumKind:
		typ = structField(g.QualifiedGoIdent(field.Enum.GoIdent))
	case protoreflect.Int32Kind, protoreflect.Sint32Kind, protoreflect.Sfixed32Kind:
		typ = "int32"
	case protoreflect.Uint32Kind, protoreflect.Fixed32Kind:
		typ = "uint32"
	case protoreflect.Int64Kind, protoreflect.Sint64Kind, protoreflect.Sfixed64Kind:
		typ = "int64"
	case protoreflect.Uint64Kind, protoreflect.Fixed64Kind:
		typ = "uint64"
	case protoreflect.FloatKind:
		typ = "float32"
	case protoreflect.DoubleKind:
		typ = "float64"
	case protoreflect.StringKind:
		typ = "string"
	case protoreflect.BytesKind:
		typ = "[]byte"
	case protoreflect.MessageKind:
		typ = structField(g.QualifiedGoIdent(field.Message.GoIdent))
		if !field.Desc.IsList() {
			typ = typ.PointerTo()
		}
	case protoreflect.GroupKind:
		panic("unimplemented")
	}

	if field.Desc.IsList() {
		typ = "[]" + typ
	}

	return typ
}
